from zope.app.form.browser.itemswidgets import DropdownWidget
from zope.component import queryMultiAdapter
from zope.schema.interfaces import ITitledTokenizedTerm

from zope.i18n.interfaces import IUserPreferredLanguages
from zope.i18n.locales import locales, LoadLocaleError

class LanguageDropdownChoiceWidget(DropdownWidget):
    """ A DropdownWidget which renders a localized language selection.
    """

    def __init__(self, field, request):
        """Initialize the widget."""
        self.request = request
        super(LanguageDropdownChoiceWidget, self).__init__(field,
            field.vocabulary, request)
        self.languages = self.locale().displayNames.languages

    def locale(self):
        envadapter = IUserPreferredLanguages(self.request, None)
        if envadapter is None:
            return None

        _locale = None
        langs = envadapter.getPreferredLanguages()
        for httplang in langs:
            parts = (httplang.split('-') + [None, None])[:3]
            try:
                _locale = locales.getLocale(*parts)
                break
            except LoadLocaleError:
                # Just try the next combination
                pass
        else:
            # No combination gave us an existing locale, so use the default,
            # which is guaranteed to exist
            _locale = locales.getLocale(None, None, None)

        return _locale

    def textForValue(self, term):
        """Extract a string from the `term`.

        The `term` must be a vocabulary tokenized term.
        """
        if ITitledTokenizedTerm.providedBy(term):
            return self.languages.get(term.value, term.title)
        return term.token
